/**
 * Copyright (C) 2003-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * @file	fpdf_layer_w.h
 * @brief	Header file for \ref FPDFPAGEOBJECTS "PDF Page Objects" module of Foxit PDF SDK.
 * @details	This header file defines PDF layer related functionalities.<br>
 *			It contains:<br>
 *			<ul>
 *			<li>1. Set the name of a layer. </li>
 *			</ul>
 *
 * @note	If you want to purchase Foxit PDF SDK license and use ANY of the following functions, please request for enabling <b>Edit Add-on</b> explicitly.
 */

#ifndef _FSPDF_LAYER_W_H_
#define _FSPDF_LAYER_W_H_

/**
 * @defgroup	FPDFPAGEOBJECTS PDF Page Objects
 * @brief		Definitions for access to PDF Page Objects, PDF Marked Content, and PDF Layers.<br>
 *				Definitions and functions in this module are included in fpdf_pageobjects_r.h, fpdf_pageobjects_w.h, fpdf_markedcontent_r.h, fpdf_markedcontent_w.h, fpdf_layer_r.h, and fpdf_layer_w.h.<br>
 *				Module: PDFPageObjects<br>
 *				License Identifier: PDFPageObjects/All<br>
 *				Available License Right: Reading/Writing<br>
 *				For License Right <b>Reading</b>, see fpdf_pageobjects_r.h, fpdf_markedcontent_r.h, and fpdf_layer_r.h.<br>
 *				For License Right <b>Writing</b>, see fpdf_pageobjects_w.h, fpdf_markedcontent_w.h, and fpdf_layer_w.h.
 * @details		This module contains following features:<br>
 *				<ul>
 *				<li>1. Page objects:
 *					<ul>
 *					<li>a. Enumerate page objects, get page objects by position, insert or remove page objects, and generate contents of page objects.</li>
 *					<li>b. Page content consists of page objects. Each page object contains its state information, data and instructions for rendering.
 *						   A form XObject is not only a page object, but also a container. A form XObject consists of a set of page objects or sub Form XObjects.</li>
 *					<li>c. Note: PDF page should be parsed before page objects can be accessed.</li>
 *					<li>d. Reference: Chapter 4 and 5 in PDF Reference 1.7 describe how page contents are expressed.</li>
 *					</ul>
 *				</li>
 *				<li>2. Page object: get type of a page object, clone a page object, and access properties or state data.</li>
 *				<li>3. Text object: get or set text state, access unicode string of a text object, and create a text object.</li>
 *				<li>4. Path object: get or set path data, access fill mode and stroke state, and create a path object.</li>
 *				<li>5. Image object: clone bitmap object associated to an image object, create an image object, and set image data.</li>
 *				<li>6. Form XObject object: get page objects associated to a Form XObject object, and create a Form XObject object.</li>
 *				<li>7. Clipping data:
 *					<ul>
 *					<li>a. Enumerate clipping path objects or clipping text objects, add or delete clipping path and text object,
 *						   and clear all clipping data.</li>
 *					<li>b. Each page object may have its own clipping data. In PDF, clipping data may be path objects or text objects.
 *						   Multiple clipping objects form intersections or union results.</li>
 *					</ul>
 *				</li>
 *				<li>8. PDF marked content:
 *					<ul>
 *					<li>a. Access to PDF page object's marked content, retrieve marked content object, add a new marked content item and delete a marked content item..</li>
 *					<li>b. Reference: Section 10.5 in PDF Reference 1.7 describes Marked Content of PDF Content Stream.</li>
 *					</ul>
 *				</li>
 *				<li>9. PDF Layer Context access: create and release PDF layer context, merge or copy state of PDF layer context objects. </li>
 *				<li>10. PDF Layer access:
 *					<ul>
 *					<li>a. Enumerate all layers of a PDF document , get and set name of a PDF layer, set and get visibility of a PDF layer. </li>
 *					<li>b. Note: If a PDF layer is invalid, it cannot be viewed, designed, printed or exported.</li>
 *					</ul>
 *				</li>
 *				<li>11. Note: most functions in this module are not thread safe, caller should ensure not to call them
 *					   to process page objects in a same page across multi-threads or should maintain thread safety by application self.</li>
 *				</ul>
 */
/**@{*/


#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************/
/* Access PDF Layer Properties                                                 */
/*******************************************************************************/

/**
 * @brief	Set the name of a PDF layer.
 *
 * @param[in]	layer		Handle to a <b>FSPDF_LAYER</b> object which is a PDF layer. It should be valid.
 * @param[in]	name		Pointer to a ::FSCRT_BSTR structure that specifies a layer name. It should be a valid UTF-8 string.<br>
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <i>layer</i> or <i>name</i> is <b>NULL</b> pointer or <i>name</i> is empty.<br>
 *			::FSCRT_ERRCODE_FORMAT if parameter <i>name</i> is not in UTF-8 format.<br>
 * 			::FSCRT_ERRCODE_INVALIDLICENSE if current license is not authorized or accessing layer not allowed.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 * 			::FSCRT_ERRCODE_OUTOFMEMORY if there is not enough memory or invalid memory access.<br>
 *			::FSCRT_ERRCODE_ERROR if it fails to set name of a PDF layer.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term recoverable.</li>
 *				<li> <i>layer</i>: this handle is long-term recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT	FSPDF_Layer_SetName(FSPDF_LAYER layer, const FSCRT_BSTR* name);

/**
 * @brief	Set the base visible state of PDF layers in the default configuration.
 *
 * @param[in]	doc			Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It shall be valid.
 * @param[in]	state		The state of a PDF layer. Please refer to macro definitions {@link FSPDF_LAYER_BASESTATE_ON FSPDF_LAYER_BASESTATE_XXX} and this should be one of these macros.
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <i>doc</i> is <b>NULL</b>, or parameter <i>state</i> is invalid.<br>
 *			::FSCRT_ERRCODE_INVALIDTYPE if parameter <i>doc</i> is not a pdf document.<br>
 * 			::FSCRT_ERRCODE_INVALIDLICENSE if current license is not authorized or accessing annotation not allowed.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 *			::FSCRT_ERRCODE_ERROR if cannot set the base state.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @note	When a document is first opened, its optional content groups are assigned a state based on the D (default) configuration dictionary in the OCProperties dictionary:
 *			1. The value of BaseState is applied to all the groups.
 *			2. The groups listed in either the ON or OFF array (depending on which one is opposite to BaseState) have their states adjusted.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term unrecoverable.</li>
 *				<li> <i>doc</i>: this handle is long-term partially recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT	FSPDF_Doc_SetLayersBaseState(FSCRT_DOCUMENT doc, FS_INT32 state);

/**
 * @brief	Set the application usage of a PDF layer. If some events is triggered, the visibility of PDF layer will be switched.
 *
 * @param[in]	layer		Handle to a <b>FSPDF_LAYER</b> object.<br>
 * @param[in]	usageType	A ::FS_INT32 object indicates the type of usage.<br>
 *							It should be one of macro definations below(not support ::FSPDF_LAYERUSAGETYPE_DESIGN):
 *							<ul>
 *							<li> ::FSPDF_LAYERUSAGETYPE_VIEW</li>
 *							<li> ::FSPDF_LAYERUSAGETYPE_PRINT.</li>
 *							<li> ::FSPDF_LAYERUSAGETYPE_EXPORT.</li>
 *							<li> ::FSPDF_LAYERUSAGETYPE_ZOOM.</li>
 *							</ul>
 * @param[out]	param		A pointer to the param of <b>eventType</b>. Its type is decided by <b>usageType</b>:
 *							<ul>
 *							<li> ::FS_BOOL,							if <b>usageType</b> is ::FSPDF_LAYERUSAGETYPE_VIEW.</li>
 *							<li> ::FSPDF_LAYERUSAGE_PRINTDATA,		if <b>usageType</b> is ::FSPDF_LAYERUSAGETYPE_PRINT.</li>
 *							<li> ::FS_BOOL,							if <b>usageType</b> is ::FSPDF_LAYERUSAGETYPE_EXPORT.</li>
 *							<li> ::FSPDF_LAYERUSAGE_ZOOMDATA,		if <b>usageType</b> is ::FSPDF_LAYERUSAGETYPE_ZOOM.</li>
 *							</ul>
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <b>layer</b> is <b>NULL</b>, <i>usageType</i> is out of range, or <b>param</b> is invalid.<br>
 *			::FSCRT_ERRCODE_INVALIDLICENSE if current license is not authorized or accessing annotation not allowed.<br>
 * 			::FSCRT_ERRCODE_OUTOFMEMORY if there is not enough memory or invalid memory access.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 *			::FSCRT_ERRCODE_ERROR if it fails to set usage of a PDF layer.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term unrecoverable.</li>
 *				<li> <i>layer</i>: this handle is long-term recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT	FSPDF_Layer_SetApplicationUsage(FSPDF_LAYER layer, FS_INT32 usageType, FS_LPVOID param);

/**
 * @brief	Remove the application usage of a PDF layer. If some events is triggered, the visibility of PDF layer will be switched.
 *
 * @param[in]	layer		Handle to a <b>FSPDF_LAYER</b> object.<br>
 * @param[in]	usageType	A ::FS_INT32 object indicates the type of usage. It should be one of macro definations below:<ul>
 *							<li> ::FSPDF_LAYERUSAGETYPE_VIEW.</li>
 *							<li> ::FSPDF_LAYERUSAGETYPE_PRINT.</li>
 *							<li> ::FSPDF_LAYERUSAGETYPE_EXPORT.</li>
 *							<li> ::FSPDF_LAYERUSAGETYPE_ZOOM.</li>
 *							</ul>
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <b>layers</b> is <b>NULL</b>, or <i>usageType</i> is out of range.<br>
 * 			::FSCRT_ERRCODE_INVALIDLICENSE if current license is not authorized or accessing annotation not allowed.<br>
 *			::FSCRT_ERRCODE_NOTFOUND if layer do not contain this usage specified by <i>usageType</i>.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 *			::FSCRT_ERRCODE_ERROR if it fails to set usage of a PDF layer.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term unrecoverable.</li>
 *				<li> <i>layer</i>: this handle is long-term recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT	FSPDF_Layer_RemoveApplicationUsage(FSPDF_LAYER layer, FS_INT32 usageType);

/**
 * @brief	Add a layer to a PDF document.
 *
 * @param[in]	doc			Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It shall be valid.
 * @param[in]	layerName	Pointer to a ::FSCRT_BSTR structure which specify the name of form field. It should be a UTF-8 string.
 * @param[out]	layer		Handle to a <b>FSPDF_LAYER</b> object which is a PDF layer.
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <i>doc</i>, <i>layer</i> or <i>layerName</i> <b>NULL</b>.<br>
 *			::FSCRT_ERRCODE_FORMAT if parameter <i>layerName</i> is not in UTF-8 format.<br>
 * 			::FSCRT_ERRCODE_INVALIDLICENSE if current license is not authorized or accessing annotation not allowed.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 * 			::FSCRT_ERRCODE_OUTOFMEMORY if there is not enough memory or invalid memory access.<br>
 *			::FSCRT_ERRCODE_ERROR if fails to add a layer.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term unrecoverable.</li>
 *				<li> <i>doc</i>: this handle is long-term partially recoverable.</li>
 *				<li> <i>layer</i>: this handle is long-term recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT FSPDF_Doc_AddLayer(FSCRT_DOCUMENT doc, FSCRT_BSTR* layerName, FSPDF_LAYER* layer);

/**
 * @brief	Remove a layer from the document.
 *
 * @details	It will reconstruct the structure of layer tree to remove a layer. Currently, Foxit SDK will remove the
 *			sub layers of the layers which is removed.
 *
 * @param[in]	layer		Handle to a <b>FSPDF_LAYER</b> object which is a PDF layer.<br>
 * @param[in]	options		Indicates how to deal with sub layers. This parameter is reserved, and should be 0.<br>
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <i>doc</i> is <b>NULL</b> pointer.<br>
 * 			::FSCRT_ERRCODE_INVALIDLICENSE if current license is not authorized or accessing annotation not allowed.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 *			::FSCRT_ERRCODE_ERROR if cannot remove the layer.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term unrecoverable.</li>
 *				<li> <i>layer</i>: this handle is long-term recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT FSPDF_Layer_Remove(FSPDF_LAYER layer, FS_DWORD options);

/**
 * @brief	Set the default visible state of a PDF layer in the default configuration.
 *
 * @param[in]	layer	Handle to a <b>FSPDF_LAYER</b> object which is a PDF layer.<br>
 * @param[in]	state	A boolean value that indicate the default status(TRUE for visible, FALSE for invisible) of a PDF layer in the document.<br>
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <i>doc</i> is <b>NULL</b> pointer.<br>
 * 			::FSCRT_ERRCODE_INVALIDLICENSE if current license is not authorized or accessing annotation not allowed.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 *			::FSCRT_ERRCODE_ERROR if cannot set the default state.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term unrecoverable.</li>
 *				<li> <i>layer</i>: this handle is long-term recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT	FSPDF_Layer_SetDefaultState(FSPDF_LAYER layer, FS_BOOL state);

/**
 * @brief	Add a page object to a PDF layer.
 *
 * @param[in]	layer		Handle to a <b>FSPDF_LAYER</b> object which is a PDF layer. It shall be valid.
 * @param[in]	page		Handle to a <b>FSCRT_PAGE</b> object which is a page object.
 * @param[in]	pageObj		Handle to a <b>FSPDF_PAGEOBJECT</b> object which is a page object.
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <i>layer</i>, <i>pageObj</i> or <i>page</i> is <b>NULL</b>.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 *			::FSCRT_ERRCODE_ERROR if it fails to add page object.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @note	In order to make sure this function works well, the <b>pageObj</b> should be on the <b>page</b>.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is short-term.</li>
 *				<li> <i>layer</i>: this handle is long-term recoverable.</li>
 *				<li> <i>page</i>: this handle is long-term partially recoverable.</li>
 *				<li> <i>pageObj</i>: this handle is short-term.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT	FSPDF_Layer_AddPageObject(FSPDF_LAYER layer, FSCRT_PAGE page, FSPDF_PAGEOBJECT pageObj);

/**
 * @brief	Remove a page object from a PDF layer.
 *
 * @param[in]	layer		Handle to a <b>FSPDF_LAYER</b> object which is a PDF layer. It shall be valid.
 * @param[in]	pageObj		Handle to a <b>FSPDF_PAGEOBJECT</b> object which is a page object.
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <i>layer</i> or <i>pageObj</i> is <b>NULL</b>.<br>
 *			::FSCRT_ERRCODE_NOTFOUND if <i>pageObj</i> is not in the layer.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if an unrecoverable error occurs.<br>
 *			::FSCRT_ERRCODE_ERROR if it fails to remove a page object.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is short-term.</li>
 *				<li> <i>layer</i>: this handle is long-term recoverable.</li>
 *				<li> <i>pageObj</i>: this handle is short-term.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT	FSPDF_Layer_RemovePageObject(FSPDF_LAYER layer, FSPDF_PAGEOBJECT pageObj);

#ifdef __cplusplus
};
#endif

/**@}*/ /* group FPDLAYER */

#endif /* _FSPDF_LAYER_W_H_ */