#pragma once
#include "pch.h"
#include "PDFDocument.h"
#include "PDFPage.h"
#include "Util.h"
#include "PDFBase.h"

using namespace Platform;
using namespace concurrency;
using namespace FSDKDLL::FSDK;

using namespace Platform::Collections;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage::FileProperties;
using namespace Windows::Foundation;
using namespace Windows::Graphics::Imaging;

/////////////////////////////////////////
//Class FSDKDLL::FSDK::PDFDocument

IAsyncOperation<Boolean>^ PDFDocument::LoadAsync(File^ file, String^ password)
{
	auto op = create_async([=]()->task<Boolean> {
		DocHandle hDoc;
		hDoc.pointer = 0;
		if (nullptr == file)
		{
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
			return create_task([]()->bool {return false; });
		}

		FileHandle hFile = file->GetFileHandle();
		if (0 == hFile.pointer)
		{
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_ERROR, NULL);
			return create_task([]()->bool {return false; });
		}
		FS_RESULT ret = FSCRT_ERRCODE_ERROR;

		auto sdkPassword = ConvertToFoxitString(password);
		FSCRT_DOCUMENT sdkDoc;
		//Load PDF document
		ret = FSPDF_Doc_StartLoad((FSCRT_FILE)hFile.pointer, &*sdkPassword, &sdkDoc, NULL);
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);

		if (FSCRT_ERRCODE_SUCCESS == ret)
			hDoc.pointer = (int64)((FS_LPVOID)sdkDoc);

		m_pdfDocument = hDoc;
		return create_task([]()->bool {return true; });
	});

	return op;
}

IObservableVector<Object^>^ PDFDocument::SaveAsPDF(File^ file, SaveAsFlag flag)
{
	if (0 == m_pdfDocument.pointer || nullptr == file || NULL == file->GetFileHandle().pointer)
	{
		return nullptr;
	}
	//Start saving PDF file
	FSCRT_PROGRESS progress = NULL;
	FS_RESULT ret = FSPDF_Doc_StartSaveToFile((FSCRT_DOCUMENT)m_pdfDocument.pointer, (FSCRT_FILE)file->GetFileHandle().pointer, (FS_INT32)flag, &progress);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		//Continue to finish saving
		//If want to do progressive saving, please use the second parameter of FSCRT_Progress_Continue.
		//See FSCRT_Progress_Continue for more details.
		ret = FSCRT_Progress_Continue(progress, NULL);
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
		//Release saving progress object
		FSCRT_Progress_Release(progress);
	}
	IObservableVector<Platform::Object^>^ buffer = nullptr;
	if (FSCRT_ERRCODE_FINISHED == ret)
	{
		buffer = ref new Vector<Object^>();
		buffer = file->GetFileBuffer();
	}
	return buffer;
}

task<IOutputStream^> PDFDocument::WriteByteToFile(IObservableVector<Object^>^ fileBuffer, StorageFile^ file)
{
	std::shared_ptr<DataWriter^> _spDataWriter = std::make_shared<DataWriter^>(nullptr);
	if (nullptr == file || nullptr == fileBuffer)
	{
		return create_task([]()->IOutputStream^ {return nullptr; });
	}
	auto OpenOp = file->OpenAsync(FileAccessMode::ReadWrite);
	return create_task(OpenOp).then([=](IRandomAccessStream^ writeStream)
	{
		writeStream->Seek(0);
		*_spDataWriter = ref new DataWriter(writeStream);
		auto size = fileBuffer->Size;
		for (unsigned int i = 0; i < size; i++)
		{
			Platform::Array<unsigned char>^ fileContent = safe_cast<Platform::Array<unsigned char>^>(fileBuffer->GetAt(i));
			(*_spDataWriter)->WriteBytes(fileContent);
		}
		return (*_spDataWriter)->StoreAsync();
	}).then([=](unsigned int bytesStored) {
		return (*_spDataWriter)->FlushAsync();
	}).then([=](bool flushSucceeded) {
		fileBuffer->Clear();
		return (*_spDataWriter)->DetachStream();
	});
}

IAsyncOperation<Boolean>^ PDFDocument::SaveAsAsync(StorageFile^ savefile, File^ sdkfile, SaveAsFlag flag)
{
	auto op = create_async([=]()->task<Boolean> {
		if (nullptr == savefile)
		{
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
			return create_task([]()->bool {return false; });
		}
		return concurrency::create_task([=]()
		{
			IObservableVector<Object^>^ buffer = SaveAsPDF(sdkfile, flag);
			return buffer;
		})
			.then([=](Windows::Foundation::Collections::IObservableVector<Object^>^ buffer)
		{
			return WriteByteToFile(buffer, savefile);
		})
			.then([=](concurrency::task<IOutputStream^> t)
		{
			IOutputStream^ outputStream = t.get();
			if (nullptr == outputStream)
			{
				FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_ERROR, NULL);
				return false;
			}
			else
			{
				FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_SUCCESS, NULL);
				return true;
			}
		});
	});
	return op;
}

Boolean PDFDocument::Close()
{
	if (0 == m_pdfDocument.pointer)
		return false;

	FS_RESULT ret = FSPDF_Doc_Close((FSCRT_DOCUMENT)(m_pdfDocument.pointer));
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

int32 PDFDocument::GetPageCount()
{
	FS_INT32 nCount = 0;
	if (0 == m_pdfDocument.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return nCount;
	}
	FS_RESULT ret = FSPDF_Doc_CountPages(((FSCRT_DOCUMENT)m_pdfDocument.pointer), &nCount);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return nCount;
}

PDFPage^ PDFDocument::GetPage(int32 index)
{
	PDFPage^page = nullptr;
	if (0 == m_pdfDocument.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return page;
	}
	FSCRT_PAGE pageGet = NULL;
	FS_RESULT ret = FSPDF_Doc_GetPage((FSCRT_DOCUMENT)m_pdfDocument.pointer, index, &pageGet);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		PageHandle pageHandle;
		pageHandle.pointer = (int64)pageGet;
		page = ref new PDFPage(pageHandle, index);
	}

	return page;
}





