#pragma once

#ifndef _PDFDOC_H_
#define _PDFDOC_H_

#include "PDFBase.h"
#include "PDFPage.h"
using namespace Platform;

namespace FSDKDLL
{
	namespace FSDK
	{
		/**
		* @brief Class for PDF document module.
		* @note This class is a wrapper for some methods defined in fpdf_document_r.h and fpdf_document_w.h of Foxit PDF SDK(C API).
		*/
		public ref class PDFDocument sealed
		{
		public:
			DocHandle GetDocHandle()
			{
				return m_pdfDocument;
			}
			

			/**
			* @brief	Load PDF document asynchronously.
			*
			* @param[in]	file		A File object, which will be used to load the document.
			* @param[in]	password	Password for input PDF document. A null value means no password.
			*
			* @return	A document handle.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of the process to loading a PDF document in Foxit PDF SDK(C API).
			*/
			IAsyncOperation<Boolean>^ LoadAsync(File^ file, String^ password); //FSPDF_Doc_StartLoad(FSCRT_FILE file, const FSCRT_BSTR* password, FSCRT_DOCUMENT* document, FSCRT_PROGRESS* progress);

			/**
			* @brief	Save current PDF document to another one, asynchronously.
			*
			* @param[in, out]	savefile		File data buffer, which is to be saved out to <i>file</i>.
			* @param[in]		sdkfile			A File object, which will be used to save the document data to file.
			* @param[in]		flag			Saving flag. Please refer to enum DemoRTSDK::Foxit::SaveAsFlag.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of the process to saving a PDF document in Foxit PDF SDK(C API).
			*/
			IAsyncOperation<Boolean>^ SaveAsAsync(StorageFile^ savefile, File^ sdkfile, SaveAsFlag flag);//FSPDF_Doc_StartSaveToFile(FSCRT_DOCUMENT document, FSCRT_FILE file, FS_INT32 flag, FSCRT_PROGRESS* saveProgress);

			/**
			* @brief	Close loaded document.
			*			
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Doc_Close in Foxit PDF SDK(C API).
			*/
			Boolean	Close();

			/**
			* @brief	Count pages.
			*			
			* @return	Count of pages. If an error occurs, it will return 0.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Doc_CountPages in Foxit PDF SDK(C API).
			*/
		    int32 GetPageCount();

			/**
			* @brief	Get a PDF page and get the page handle.
			*
			* @param[in]	index		Index of PDF page. Range: 0 to (pagecount-1). <i>pagecount</i> is returned by function FSDKDLL::FSDK::PDFDocument::GetPageCount.
			*
			* @return	PDFPage
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Doc_GetPage in Foxit PDF SDK(C API).
			*/
			PDFPage^ PDFDocument::GetPage(int32 index);
		private:
			/**
			* @brief	Write file data(in byte) to file.
			*
			* @details	This is a help function, for internal use, mainly for SaveAsAsync().
			*			In this function, no use of Foxit PDF SDK(C API).
			*
			* @param[in]		fileBuffer		File data buffer, which is to be saved out to <i>file</i>.
			* @param[in, out]	file			A StorageFile object, which represents the saved file.
			*
			* @return	A IOutputStream object
			*
			*/
			static Concurrency::task<Windows::Storage::Streams::IOutputStream^> WriteByteToFile(Windows::Foundation::Collections::IObservableVector<Object^>^ fileBuffer,
				Windows::Storage::StorageFile^ file);
			/**
			* @brief	Save current PDF file as a new one.
			*
			* @details	This is an internal used function, mainly for SaveAsAsync().
			*			In this function, it wraps the progress about how to using Foxit PDF SDK(C API) to save current PDF file as another one.
			*
			* @param[in]		document		A Document handle, which represents current opened PDF file.
			* @param[in]		file			A File object, which will be used to save the document data to file.
			* @param[in]		flag			Saving flag. Please refer to enum DemoRTSDK::Foxit::SaveAsFlag.
			*
			* @return	FSCRT_ERRCODE_SUCCESS means success.
			*			For more error codes, please refer to macro definitions <b>FSCRT_ERRCODE_XXX</b> in Foxit PDF SDK.
			*
			*/
			Windows::Foundation::Collections::IObservableVector<Object^>^ SaveAsPDF(File^ file, SaveAsFlag flag);

			property DocHandle m_pdfDocument;
		};
	}
}

#endif //_PDFDOC_H_