#pragma once

#ifndef _PDFPAGE_H_
#define _PDFPAGE_H_

#include "PDFBase.h"
#include "PDFText.h"
#include "PDFAnnot.h"

using namespace Platform;
using namespace Platform::Collections;

namespace FSDKDLL
{
	namespace FSDK
	{
		/**
		* @brief Class for PDF page module.
		* @note This class is a wrapper for some methods defined in fpdf_page_r.h and fpdf_document_r.h of Foxit PDF SDK(C API).
		*/
		public ref class PDFPage sealed
		{
		public:
			PDFPage(PageHandle pageHandle, int pageIndex)
			{
				m_pdfPage = pageHandle;
				m_nPageIndex = pageIndex;
			}

			PageHandle GetPageHandle()
			{
				return m_pdfPage;
			}

			/**
			* @brief	Release all page contents and relative resources
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Page_Clear in Foxit PDF SDK(C API).
			*/
			Boolean Close();

			/**
			* @brief	Paring a page handle
			*
			* @param[in]	flag	Parse flag.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of the process to parse a PDF page in Foxit PDF SDK(C API).
			*/
			Boolean Parse(PageParseFlag flag);//without pause yet.Pause^ pause //FSPDF_Page_StartParse(FSCRT_PAGE page, FS_DWORD flag, FSCRT_PROGRESS* parseProgress)

			/**
			* @brief	Render page asynchronously.
			*
			* @param[in,out]	pxsrc		A PixelSource object used to receive the rendering result data.
			* @param[in]		iStartX		Left position of the transformation area in device coordinate, a ::FS_INT32 value, commonly in pixels.
			* @param[in]		iStartY		Top position of the transformation area in device coordinate, a ::FS_INT32 value, commonly in pixels.
			* @param[in]		iSizeX		Width of the transformation area in device coordinate, a ::FS_INT32 value, commonly in pixels.s
			* @param[in]		iSizeY		Height of the transformation area in device coordinate, a ::FS_INT32 value, commonly in pixels.
			* @param[in]		rotation	Page rotation for rendering.
			*
			* @return	A IRandomAccessStreamWithContentType object, which can be used in demo for showing page to device.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of the process to render a PDF page in Foxit PDF SDK(C API).
			*/
			IAsyncOperation<IRandomAccessStreamWithContentType^>^ RenderPageAsync(PixelSource^ pxsrc, int32 iStartX, int32 iStartY,
				int32 iSizeX, int32 iSizeY, PageRotation rotation);

			/**
			* @brief	Get the width of page.
			*
			* @return	The width value.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Page_GetSize in Foxit PDF SDK(C API).
			*/
			float32 GetWidth();

			/**
			* @brief	Get the height of page
			*
			* @return	The height value.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Page_GetSize in Foxit PDF SDK(C API).
			*/
			float32 GetHeight();

			/**
			* @brief	Load annots of page.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Page_LoadAnnots in Foxit PDF SDK(C API).
			*/
			Boolean LoadAnnots();

			/**
			* @brief	Unload annotations of a PDF page.
			*
			* @details	This should be called before any other functions defined in this class.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Page_UnloadAnnots in Foxit PDF SDK(C API).
			*/
			Boolean UnLoadAnnots();

			/**
			* @brief	Get an annotation with a specific index, by specific filter.
			*
			* @param[in] filter		Vector of filter . If it's null, parameter <i>index</i> is to all existing annotation.<br>
			* @param[in] index		A zero-based index of annotation to be got. Range: 0 to (annotcount-1).
			*
			* @return	An annot handle.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Annot_Get in Foxit PDF SDK(C API).
			*/
			PDFAnnot^ GetAnnot(IVector<AnnotType>^ filter, int32 index);

			/**
			* @brief	Get count of annotations, by specific filter.
			*
			* @param[in] filter		Vector of annottype. If it's null, this function will count all annotations.<br>
			*
			* @return	Count of annotations. If there's any error, it will return 0.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Annot_GetCount in Foxit PDF SDK(C API).
			*/
			int32 GetAnnotsCount(IVector<AnnotType>^ filter);

			/**
			* @brief	Add an annotation with a specific index, by specific filter.
			*
			* @param[in] rectf		Reference a ref class RectF to specify the rectangle of an annotation, in PDF page coordinate system.
			* @param[in] annottype	Specify the type of an annotation. This should be in the type range defined by parameter <i>filter</i>.
			* @param[in] filter		Vector of filter. If it's null, parameter <i>index</i> is to all existing annotation.<br>
			* @param[in] index		The index that this new annotation is to be added in. Range: 0 to (annotcount-1).
			*
			* @return	An annot handle.
			*			When failed, please call DemoRTSDK::Foxit::Base::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Annot_Add in Foxit PDF SDK(C API).
			*/
			PDFAnnot^ AddAnnot(RectF^ rectf, AnnotType annottype, IVector<AnnotType>^ filter, int32 index);

			/**
			* @brief Prepare the information about all characters in a page.
			*
			* @return	PDFText.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_TextPage_Load in Foxit PDF SDK(C API).
			*/
			PDFText^ LoadTextPage();

			Boolean TransformPageMatrixToPointF(int32 pageX, int32 pageY, int32 pageWidth, int32 pageHeight, int32 rotation, PointF^ pointF);

			RectF^ TransformPageMatrixToRectF(int32 pageX, int32 pageY, int32 pageWidth, int32 pageHeight, int32 rotation, RectF^ rectF);

			property int32 m_nPageIndex;
		private:
			//Render page to bitmap and get its data.
			Boolean GetRenderBitmapData(PixelSource^ pxsrc, PageHandle hPage, int iStartX, int iStartY, int iSizeX, int iSizeY, int iRotation);
			//Get bitmap's data
			FS_RESULT GetBitmapData(FSCRT_BITMAP bmp, PixelSource^ dib);
			//Render page to SDK bitmap
			FS_RESULT PDFPageToBitmap(FSCRT_PAGE page, int bmpWidth, int bmpHeight, int iStartX, int iStartY, int iSizeX, int iSizeY, int iRotation, FSCRT_BITMAP *renderBmp);

			property PageHandle m_pdfPage;
		};
	}
}
#endif //_PDFPAGE_H_
